"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  MCHelper: () => MCHelper
});
module.exports = __toCommonJS(src_exports);

// src/utils.ts
function error(error2, ...args) {
  if (typeof error2 === "string") {
    console.error(`[MCHE Error] ${error2}`, ...args);
    return;
  }
  console.error(`[MCHE Error] ${error2.message}`);
  console.error(error2.stack);
}
function log(...args) {
  console.log("[MCHE Log]", ...args);
}
function registerOnWsConnected(ws, id, roomId) {
  ws.registerCallbacks({
    onConnected: () => {
      ws.ws.send("register", JSON.stringify({
        roomId,
        userId: id
      }));
    }
  });
}

// src/webrtc/error-handling.ts
var sctpCauseCodes = [
  "No SCTP error",
  "Invalid stream identifier",
  "Missing mandatory parameter",
  "Stale cookie error",
  "Sender is out of resource (i.e., memory)",
  "Unable to resolve address",
  "Unrecognized SCTP chunk type received",
  "Invalid mandatory parameter",
  "Unrecognized parameters",
  "No user data (SCTP DATA chunk has no data)",
  "Cookie received while shutting down",
  "Restart of an association with new addresses",
  "User-initiated abort",
  "Protocol violation"
];
function handleDCError(ev, connection) {
  const err = ev.error;
  error("WebRTC error: ", err.message);
  switch (err.errorDetail) {
    case "sdp-syntax-error":
      error("    SDP syntax error in line ", err.sdpLineNumber);
      break;
    case "sctp-failure": {
      if (err.sctpCauseCode && err.sctpCauseCode < sctpCauseCodes.length)
        error("    SCTP failure: ", err.sctpCauseCode);
      else
        error("    Unknown SCTP error");
      break;
    }
    case "dtls-failure":
      if (err.receivedAlert)
        error("    Received DLTS failure alert: ", err.receivedAlert);
      if (err.sentAlert)
        error("    Sent DLTS failure alert: ", err.receivedAlert);
      break;
  }
  connection.close();
}

// src/webrtc/connection.ts
var ICE_SERVERS = [
  {
    urls: "stun:stun.l.google.com:19302"
  }
];
var WebRtcConnection = class {
  #peerConnection;
  #datachannel = null;
  #datachannelCallbacks = [];
  #datachannelReadyCallbacks = [];
  #debug;
  constructor(onDataChannelMessage, debug, customIceServers) {
    this.#peerConnection = new RTCPeerConnection({
      iceServers: [
        ...customIceServers ?? [],
        ...ICE_SERVERS
      ]
    });
    this.#peerConnection.ondatachannel = (event) => {
      const channel = this.#datachannel = event.channel;
      if (this.#debug)
        log("RemoteDataChannel created.", channel);
      this.#datachannelCallbacks.forEach((callback) => {
        callback(channel);
      });
    };
    this.#datachannelCallbacks.push((channel) => {
      channel.onmessage = (event) => {
        onDataChannelMessage(event);
      };
      channel.onopen = () => {
        this.#datachannelReadyCallbacks.forEach((callback) => {
          callback(channel);
        });
      };
      channel.onerror = (event) => {
        handleDCError(event, this);
      };
    });
    this.#debug = debug;
  }
  async createOffer() {
    const channel = this.#datachannel = this.#peerConnection.createDataChannel("datachannel");
    if (this.#debug)
      log("LocalDataChannel created.", this.#datachannel);
    this.#datachannelCallbacks.forEach((callback) => {
      callback(channel);
    });
    const offer = await this.#peerConnection.createOffer();
    await this.#peerConnection.setLocalDescription(offer);
    return offer;
  }
  async receiveOffer(offer) {
    await this.#peerConnection.setRemoteDescription(offer);
  }
  async createAnswer() {
    const answer = await this.#peerConnection.createAnswer();
    await this.#peerConnection.setLocalDescription(answer);
    return answer;
  }
  async receiveAnswer(answer) {
    await this.#peerConnection.setRemoteDescription(answer);
  }
  async addIceCandidate(candidate) {
    await this.#peerConnection.addIceCandidate(candidate);
  }
  async createDataChannel(label) {
    const channel = this.#peerConnection.createDataChannel(label);
    return channel;
  }
  onCandidate(callback) {
    this.#peerConnection.onicecandidate = (event) => {
      if (event.candidate)
        callback(event.candidate);
    };
  }
  addDataChannelCallback(callback) {
    if (this.#datachannel) {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelCallbacks.push(callback);
  }
  addDataChannelReadyCallback(callback) {
    if (this.#datachannel?.readyState === "open") {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelReadyCallbacks.push(callback);
  }
  close() {
    this.#peerConnection.close();
    this.#datachannel?.close();
    this.#datachannelCallbacks.length = 0;
  }
};

// src/ws.ts
var import_wshe = require("wshe");
var WsClient = class {
  #ws;
  #roomId;
  #onConnectedCallbacks = [];
  #onDisconnectedCallbacks = [];
  #onErrorCallbacks = [];
  constructor(urlOrWsInstance, roomId) {
    this.#ws = (0, import_wshe.createWSHE)(urlOrWsInstance, {
      immediate: true,
      onConnected: (ws, event) => {
        this.#onConnectedCallbacks.forEach((callback) => callback(ws, event));
      },
      onDisconnected: (ws, event) => {
        this.#onDisconnectedCallbacks.forEach((callback) => callback(ws, event));
      },
      onError: (ws, event) => {
        this.#onErrorCallbacks.forEach((callback) => callback(ws, event));
      }
    });
    this.#roomId = roomId;
  }
  send(topic, data) {
    this.#ws.send(topic, {
      ...data,
      roomId: this.#roomId
    });
  }
  sendRaw(data) {
    this.#ws.sendRaw(data);
  }
  onRaw(callback) {
    return this.#ws.subscribeRaw(callback);
  }
  on(topic, callback) {
    this.#ws.subscribe(topic, (data) => {
      if (data.roomId !== this.#roomId)
        return;
      callback(data);
    });
  }
  close() {
    this.#onConnectedCallbacks.length = 0;
    this.#onDisconnectedCallbacks.length = 0;
    this.#onErrorCallbacks.length = 0;
    this.#ws.close();
  }
  get ws() {
    return {
      ws: this.#ws,
      registerCallbacks: (callbacks) => {
        if (callbacks.onConnected)
          this.#onConnectedCallbacks.push(callbacks.onConnected);
        if (callbacks.onDisconnected)
          this.#onDisconnectedCallbacks.push(callbacks.onDisconnected);
        if (callbacks.onError)
          this.#onErrorCallbacks.push(callbacks.onError);
      }
    };
  }
};

// src/base.ts
var AbstractContainer = class {
};

// src/webrtc/index.ts
var WebRtcContainer = class extends AbstractContainer {
  #id;
  #options;
  #signalingServer;
  #connectionPool = /* @__PURE__ */ new Map();
  #dataBuffer = [];
  #dataBufferCallbacks = [];
  #onMessageChannelReadyCallbacks = [];
  #onJoinCallbacks = [];
  #onLeaveCallbacks = [];
  constructor(options) {
    super();
    this.#id = options.id;
    this.#signalingServer = new WsClient(options.signalingServerUrlOrWsInstance, options.roomId);
    this.#options = options;
    this.#bindEvents().catch(error);
    this.#bindSignalingEvents();
    registerOnWsConnected(this.#signalingServer.ws, this.#id, this.#options.roomId);
  }
  #addMessageToBuffer(message) {
    this.#dataBuffer.push(message);
    this.#dataBufferCallbacks.forEach((callback) => {
      callback(message.data);
      this.#dataBuffer.length = 0;
    });
  }
  #getConnection() {
    return new WebRtcConnection((message) => {
      this.#addMessageToBuffer(message);
    }, this.#options.debug, this.#options.iceServers);
  }
  #addNewConnection(id, conn) {
    this.#connectionPool.set(id, conn);
    this.#onMessageChannelReadyCallbacks.forEach((callback) => {
      conn.addDataChannelReadyCallback(callback);
    });
  }
  async #bindEvents() {
    this.#signalingServer.on("mche:offer:send" /* OfferSend */, async ({ receiver, offer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received offer from", sender, "payload is", offer);
      let connection = this.#connectionPool.get(sender);
      if (!connection) {
        connection = this.#getConnection();
        this.#addNewConnection(sender, connection);
      }
      await connection.receiveOffer(offer);
      const answer = await connection.createAnswer();
      const payload = {
        answer,
        // swap sender and receiver
        receiver: sender,
        sender: receiver
      };
      if (this.#options.debug)
        log("Sending answer to", sender, "payload is", payload);
      this.#signalingServer.send("mche:answer:send" /* AnswerSend */, payload);
    });
    this.#signalingServer.on("mche:answer:send" /* AnswerSend */, async ({ receiver, answer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received answer from", sender, "payload is", answer);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.receiveAnswer(answer);
    });
    this.#signalingServer.on("mche:candidate:send" /* CandidateSend */, async ({ receiver, candidate, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received candidate from", sender, "payload is", candidate);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.addIceCandidate(candidate);
    });
  }
  /**
   * Call this function to connection other peers.
   * @returns The ID of the peer. You need **hold** this ID to do next operations.
   */
  async #registerRemote(remoteId) {
    const connection = this.#getConnection();
    connection.onCandidate((candidate) => {
      const payload = {
        receiver: remoteId,
        sender: this.#id,
        candidate
      };
      if (this.#options.debug)
        log("Sending candidate to", remoteId, "payload is", payload);
      this.#signalingServer.send("mche:candidate:send" /* CandidateSend */, payload);
    });
    this.#addNewConnection(remoteId, connection);
    await this.#connect(remoteId);
    return remoteId;
  }
  async #connect(peerId) {
    const connection = this.#connectionPool.get(peerId);
    if (!connection) {
      error(new Error(`Peer ${peerId} not found.`));
      return;
    }
    const offer = await connection.createOffer();
    const payload = {
      offer,
      sender: this.#id,
      receiver: peerId
    };
    if (this.#options.debug)
      log("Sending offer to", peerId, "payload is", payload);
    this.#signalingServer.send("mche:offer:send" /* OfferSend */, payload);
  }
  #bindSignalingEvents() {
    this.#signalingServer.on("open", ({ data }) => {
      if (this.#options.debug)
        log("Open connection with", data);
      data.forEach(({ id }) => {
        this.#registerRemote(id);
      });
      this.#onJoinCallbacks.forEach((callback) => {
        callback(data);
      });
    });
    this.#signalingServer.on("close", ({ data }) => {
      const { id } = data;
      const connection = this.#connectionPool.get(id);
      if (connection) {
        if (this.#options.debug)
          log("Close connection with", id);
        connection.close();
        this.#connectionPool.delete(id);
        this.#onLeaveCallbacks.forEach((callback) => {
          callback(data);
        });
      }
    });
  }
  broadcast(message) {
    const payload = {
      sender: this.#id,
      receiver: "broadcast",
      message
    };
    this.#connectionPool.forEach((connection) => {
      connection.addDataChannelReadyCallback((channel) => {
        channel.send(payload.message);
      });
    });
    return payload;
  }
  send(targetIds, message) {
    const payload = {
      sender: this.#id,
      receiver: targetIds.toString(),
      message
    };
    const targetIdsSet = new Set(targetIds);
    this.#connectionPool.forEach((connection, id) => {
      if (targetIdsSet.has(id)) {
        connection.addDataChannelReadyCallback((channel) => {
          channel.send(payload.message);
        });
      }
    });
    return payload;
  }
  onMessageChannelReady(callback) {
    this.#onMessageChannelReadyCallbacks.push(callback);
  }
  onJoin(callback) {
    this.#onJoinCallbacks.push(callback);
    return () => {
      const index = this.#onJoinCallbacks.indexOf(callback);
      if (index !== -1)
        this.#onJoinCallbacks.splice(index, 1);
    };
  }
  onLeave(callback) {
    this.#onLeaveCallbacks.push(callback);
    return () => {
      const index = this.#onLeaveCallbacks.indexOf(callback);
      if (index !== -1)
        this.#onLeaveCallbacks.splice(index, 1);
    };
  }
  onBroadcast(callback) {
    const fn = (message) => {
      if (this.#options.debug === "verbose")
        log("Get broadcasted message", message);
      callback(message);
    };
    this.#dataBufferCallbacks.push(fn);
    return () => {
      const index = this.#dataBufferCallbacks.indexOf(fn);
      if (index !== -1)
        this.#dataBufferCallbacks.splice(index, 1);
    };
  }
  close() {
    this.#connectionPool.forEach((connection) => {
      connection.close();
    });
    this.#signalingServer.close();
    this.#onJoinCallbacks.length = 0;
    this.#onLeaveCallbacks.length = 0;
  }
};

// src/websocket/index.ts
var WebSocketContainer = class extends AbstractContainer {
  #ws;
  #onDataChannelReadyCallbacks = [];
  #onJoinCallbacks = [];
  #onLeaveCallbacks = [];
  #id;
  #connected = false;
  constructor(options) {
    super();
    this.#id = options.id;
    this.#ws = new WsClient(options.urlOrWsInstance, options.roomId);
    registerOnWsConnected(this.#ws.ws, options.id, options.roomId);
    this.#ws.on("open", ({ data, roomId }) => {
      if (roomId !== options.roomId)
        return;
      this.#onJoinCallbacks.forEach((callback) => callback(data));
    });
    this.#ws.on("close", ({ data, roomId }) => {
      if (roomId !== options.roomId)
        return;
      this.#onLeaveCallbacks.forEach((callback) => callback(data));
    });
    this.#ws.on("register_accept", ({ id, roomId }) => {
      if (roomId !== options.roomId || id !== this.#id)
        return;
      this.#connected = true;
      this.#onDataChannelReadyCallbacks.forEach((callback) => callback());
      this.#onDataChannelReadyCallbacks.length = 0;
    });
  }
  onMessageChannelReady(callback) {
    if (this.#connected)
      callback();
    else
      this.#onDataChannelReadyCallbacks.push(callback);
  }
  broadcast(message) {
    const payload = {
      sender: this.#id,
      receiver: "broadcast",
      message
    };
    if (this.#connected) {
      this.#ws.sendRaw(message);
    } else {
      this.onMessageChannelReady(() => {
        this.#ws.sendRaw(message);
      });
    }
    return payload;
  }
  onBroadcast(callback) {
    return this.#ws.onRaw(callback);
  }
  send(targetIds, message) {
    const payload = {
      sender: this.#id,
      receiver: targetIds.toString(),
      message
    };
    this.broadcast(message);
    return payload;
  }
  onJoin(callback) {
    this.#onJoinCallbacks.push(callback);
    return () => {
      const index = this.#onJoinCallbacks.indexOf(callback);
      if (index !== -1)
        this.#onJoinCallbacks.splice(index, 1);
    };
  }
  onLeave(callback) {
    this.#onLeaveCallbacks.push(callback);
    return () => {
      const index = this.#onLeaveCallbacks.indexOf(callback);
      if (index !== -1)
        this.#onLeaveCallbacks.splice(index, 1);
    };
  }
  close() {
    this.#ws.close();
    this.#connected = false;
  }
};

// src/index.ts
var MCHelper = class {
  #options;
  #id;
  #container;
  constructor(options) {
    this.#options = options;
    this.#id = options.id;
    if (options.mode === "webrtc")
      this.#container = new WebRtcContainer(options);
    else
      this.#container = new WebSocketContainer(options);
  }
  get id() {
    return this.#id;
  }
  serializeObjectPayload(payload) {
    return JSON.stringify(payload);
  }
  deserializeObjectPayload(payload) {
    return JSON.parse(payload);
  }
  broadcast(message) {
    const payload = this.#container.broadcast(message);
    if (this.#options.debug === "verbose")
      log("Broadcasting message", message, "payload is", payload);
    return payload;
  }
  send(targetIds, message) {
    const payload = this.#container.send(targetIds, message);
    if (this.#options.debug === "verbose")
      log("Sending message", message, "payload is", payload);
    return payload;
  }
  onMessageChannelReady(callback) {
    this.#container.onMessageChannelReady(callback);
  }
  onJoin(callback) {
    return this.#container.onJoin(callback);
  }
  onLeave(callback) {
    return this.#container.onLeave(callback);
  }
  onBroadcast(callback) {
    return this.#container.onBroadcast(callback);
  }
  close() {
    this.#container.close();
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  MCHelper
});
