// src/server.ts
import { getHeartbeatResponse, isHeartbeatRequestParsed, jsonStringify, tryToParseRawMessage, withSign } from "wshe/server";
function buildMetaPayload(event, data) {
  return withSign(jsonStringify({
    event,
    data
  }));
}
function withHandleMetaEvent() {
  return {
    tryParseMetaEvent(event) {
      return tryToParseRawMessage(event);
    },
    // heartbeat...
    isHeartbeatRequestParsed,
    getHeartbeatResponse,
    // register....
    isMetaRegisterEvent(event) {
      return event.event === "register" /* Register */;
    },
    getMetaRegisterEventPayload(event) {
      return event.data;
    },
    getMetaRegisterResponse(roomId, userId) {
      return buildMetaPayload("open" /* Open */, {
        roomId,
        data: [{ id: userId }]
      });
    },
    getMetaRegisterAcceptResponse(roomId, userId) {
      return buildMetaPayload("register-accept" /* RegisterAccept */, {
        roomId,
        id: userId
      });
    },
    // close...
    getMetaCloseResponse(roomId, userId) {
      return buildMetaPayload("close" /* Close */, {
        roomId,
        data: {
          id: userId
        }
      });
    }
  };
}
export {
  buildMetaPayload,
  withHandleMetaEvent
};
